
import 'dart:io';

import 'package:app_tracking_transparency/app_tracking_transparency.dart';
import 'package:chobi_admob_flutter/chobi_admob.dart';
import 'package:chobi_admob_flutter/chobi_admob_flutter_platform_interface.dart';
import 'package:chobilib_flutter/extensions.dart';
import 'package:flutter/cupertino.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

class ChobiAdMobBannerAd extends ConsumerStatefulWidget {

  final String? androidBannerId;
  final String? iosBannerId;
  final String lastClickedPrefsKey;
  final List<String>? testDeviceIds;
  final bool isDebug;
  final int adIntervalTimeMs;

  final Function(ConsentStatus status)? additionalConsentCheckFinished;
  final Function(FormError error)? additionalConsentError;

  final BannerAdListener? bannerAdListener;

  late final StateProvider<bool> isAdAvailableProvider;
  late final StateProvider<bool> isAdLoadedProvider;
  late final StateProvider<BannerAd?> bannerAdProvider;
  //final StateProvider<TrackingStatus>? appTrackingStatusProvider;

  final bool needToConfirmIosAppTracking;

  final Widget Function(BuildContext context, BannerAd? ad) builder;

  ChobiAdMobBannerAd({
    Key? key,
    this.androidBannerId,
    this.iosBannerId,
    required this.lastClickedPrefsKey,
    this.testDeviceIds,
    this.isDebug = true,
    this.adIntervalTimeMs = 5 * 60 * 1000,
    this.additionalConsentCheckFinished,
    this.additionalConsentError,
    this.bannerAdListener,
    StateProvider<bool>? isAdAvailableProvider,
    StateProvider<bool>? isAdLoadedProvider,
    StateProvider<BannerAd?>? bannerAdProvider,
    this.needToConfirmIosAppTracking = true,
    //this.appTrackingStatusProvider,
    required this.builder,
  }) : super(key: key) {
    this.isAdAvailableProvider = isAdAvailableProvider ?? StateProvider((ref) => false);
    this.isAdLoadedProvider = isAdLoadedProvider ?? StateProvider((ref) => false);
    this.bannerAdProvider = bannerAdProvider ?? StateProvider((ref) => null);
  }

  @override
  ConsumerState<ChobiAdMobBannerAd> createState() => ChobiAdMobBannerAdState();
}


const _blankBox = SizedBox.shrink(key: ValueKey('chobi_admob_banner_ad_blank_box'),);

class ChobiAdMobBannerAdState extends ConsumerState<ChobiAdMobBannerAd> with AutomaticKeepAliveClientMixin {

  late ChobiAdMob? _chobiAdMob;

  final _iosAppTrackingConfirmedProvider = StateProvider((ref) => false);

  @override
  bool get wantKeepAlive => true;

  @override
  void initState() {
    super.initState();

    WidgetsBinding.instance.addPostFrameCallback((_) async {
      if (Platform.isIOS && widget.needToConfirmIosAppTracking) {
        //final isAvailableRequest = await AppTrackingTransparency.trackingAuthorizationStatus;
        //if (isAvailableRequest == TrackingStatus.notDetermined) {
          /*
          //final status = await AppTrackingTransparency.requestTrackingAuthorization();
          widget.appTrackingStatusProvider?.also((t) {
            ref.read(t.notifier).state = status;
          });

           */

          await ChobiAdmobFlutterPlatform.instance.confirmAppTrackingTransparency();
        //}
      }
      ref.read(_iosAppTrackingConfirmedProvider.notifier).state = true;

      await ChobiAdMob.loadLastClickedTimeFromPrefs(prefKey: widget.lastClickedPrefsKey).then((value) {
        _chobiAdMob = ChobiAdMob(
          adLastClickedTime: value,
          lastClickedPrefsKey: widget.lastClickedPrefsKey,
          isDebug: widget.isDebug,
          testDeviceIds: widget.testDeviceIds,
          androidId: widget.androidBannerId,
          iosId: widget.iosBannerId,
          androidTestId: ChobiAdMob.androidTestAdaptiveBannerId,
          iosTestId: ChobiAdMob.iosTestAdaptiveBannerId,
          adIntervalTimeMs: widget.adIntervalTimeMs,
        )..onConsentCheckFinished =  (status) {
          _onConsentCheckFinished(status);
          widget.additionalConsentCheckFinished?.call(status);
        }..onConsentCheckError = (err) {
          _onConsentError(err);
          widget.additionalConsentError?.call(err);
        }..checkPermissionAndStartAdProcess();
      });
    });
  }


  @override
  void dispose() {
    Future.sync(() async {
      await _chobiAdMob?.stopAdProcess();
      await disposeBanner();
    });

    super.dispose();
  }

  @protected
  Future<void> loadBanner() async {
    if (!ref.read(widget.isAdAvailableProvider)) {
      debugPrint('AD not available');
      return;
    }

    final bannerId = _chobiAdMob?.getAdId();
    debugPrint('banner id = $bannerId');

    if (bannerId == null) {
      debugPrint('banner id is null');
      return;
    }

    final cWidth = MediaQuery.of(context).size.width.truncate();
    final adSize = await AdSize.getCurrentOrientationAnchoredAdaptiveBannerAdSize(cWidth);

    if (adSize == null) {
      debugPrint('adSize is null');
      return;
    }

    final ad = BannerAd(
      size: adSize,
      adUnitId: bannerId,
      listener: generateBannerListener(),
      request: const AdRequest(),
    );

    ref.read(widget.bannerAdProvider.notifier).state = ad;

    await ad.load();
  }

  @protected
  Future<void> disposeBanner() async {
    await ref.read(widget.bannerAdProvider)?.dispose();
    ref.read(widget.bannerAdProvider.notifier).state = null;
    ref.read(widget.isAdAvailableProvider.notifier).state = false;
    ref.read(widget.isAdLoadedProvider.notifier).state = false;
  }

  @protected
  BannerAdListener generateBannerListener() {
    return BannerAdListener(
        onAdClosed: widget.bannerAdListener?.onAdClosed,
        onAdImpression: widget.bannerAdListener?.onAdImpression,
        onAdOpened: widget.bannerAdListener?.onAdOpened,
        onAdWillDismissScreen: widget.bannerAdListener?.onAdWillDismissScreen,
        onPaidEvent: widget.bannerAdListener?.onPaidEvent,

        onAdLoaded: (ad) {
          //ref.read(widget.bannerAdProvider.notifier).state = ad as BannerAd?;
          ref.read(widget.isAdLoadedProvider.notifier).state = true;
          widget.bannerAdListener?.onAdLoaded?.call(ad);
        },
        onAdFailedToLoad: (ad, err) {
          debugPrint('admob banner laod error: ${err.message}');
          ref.read(widget.isAdLoadedProvider.notifier).state = false;
          widget.bannerAdListener?.onAdFailedToLoad?.call(ad, err);
          //ref.read(widget.bannerAdProvider.notifier).state = null;
          disposeBanner();
        },
        onAdClicked: (ad) async {
          await _chobiAdMob?.updateLastClickedTime();
          await disposeBanner();
          await _chobiAdMob?.checkPermissionAndStartAdProcess();
          widget.bannerAdListener?.onAdClicked?.call(ad);
        }
    );
  }


  void _onConsentCheckFinished(ConsentStatus status) {
    if (status != ConsentStatus.required) {
      ref.read(widget.isAdAvailableProvider.notifier).state = true;
      loadBanner();
    }
  }

  void _onConsentError(FormError error) {
    ref.read(widget.isAdAvailableProvider.notifier).state = false;
    debugPrint('UMP FORM ERROR[${error.errorCode}]: ${error.message}');
  }


  @override
  Widget build(BuildContext context) {
    super.build(context);

    return Consumer(
      builder: (atCtx, atRef, atChild) {
        final isAppTrackingChecked = atRef.watch(_iosAppTrackingConfirmedProvider);
        return isAppTrackingChecked ? Consumer(
          builder: (adCtx, adRef, adChild) {
            final ad = adRef.watch(widget.bannerAdProvider);
            return widget.builder(adCtx, ad);
          },
        ) : _blankBox;
      },
    );

    /*
    final isAppTrackingChecked = ref.watch(_iosAppTrackingConfirmedProvider);
    final ad = ref.watch(widget.bannerAdProvider);
    return isAppTrackingChecked
        ? widget.builder(context, ad)
        : const SizedBox.shrink(key: ValueKey('chobi_admob_banner_ad_blank_box'),);

     */
  }
}

